@@ -110,7 +110,7 @@ impl VisitMut for Fixer<'_> {
         self.ctx = Context::Callee { is_new: false };
         node.callee.visit_mut_with(self);
         match &mut node.callee {
-            ExprOrSuper::Expr(e) if e.is_cond() || e.is_bin() || e.is_lit() => {
+            ExprOrSuper::Expr(e) if e.is_cond() || e.is_bin() || e.is_lit() || e.is_unary() => {
                 self.wrap(&mut **e);
             }
             _ => {}
@@ -187,7 +187,9 @@ impl VisitMut for Fixer<'_> {
             // While simplifying, (1 + x) * Nan becomes `1 + x * Nan`.
             // But it should be `(1 + x) * Nan`
             Expr::Bin(BinExpr { op: op_of_lhs, .. }) => {
-                if op_of_lhs.precedence() < expr.op.precedence() {
+                if op_of_lhs.precedence() < expr.op.precedence()
+                    || (op_of_lhs.precedence() == expr.op.precedence() && expr.op == op!("**"))
+                {
                     self.wrap(&mut expr.left);
                 }
             }
@@ -275,7 +277,7 @@ impl VisitMut for Fixer<'_> {
         n.visit_mut_children_with(self);
         self.ctx = old;
 
-        match *n.arg {
+        match &*n.arg {
             Expr::Assign(..)
             | Expr::Bin(..)
             | Expr::Seq(..)
@@ -336,9 +338,11 @@ impl VisitMut for Fixer<'_> {
     }
 
     fn visit_mut_expr(&mut self, e: &mut Expr) {
+        let ctx = self.ctx;
         self.unwrap_expr(e);
         e.visit_mut_children_with(self);
 
+        self.ctx = ctx;
         self.wrap_with_paren_if_required(e)
     }
     fn visit_mut_expr_or_spread(&mut self, e: &mut ExprOrSpread) {
@@ -423,6 +427,23 @@ impl VisitMut for Fixer<'_> {
         self.ctx = old;
     }
 
+    fn visit_mut_tagged_tpl(&mut self, e: &mut TaggedTpl) {
+        e.visit_mut_children_with(self);
+
+        match &*e.tag {
+            Expr::Arrow(..)
+            | Expr::Cond(..)
+            | Expr::Bin(..)
+            | Expr::Seq(..)
+            | Expr::Fn(..)
+            | Expr::Assign(..)
+            | Expr::Unary(..) => {
+                self.wrap(&mut e.tag);
+            }
+            _ => {}
+        }
+    }
+
     fn visit_mut_module(&mut self, n: &mut Module) {
         debug_assert!(self.span_map.is_empty());
         self.span_map.clear();
@@ -1190,4 +1211,12 @@ var store = global[SHARED] || (global[SHARED] = {});
     identical!(deno_10668_1, "console.log(null ?? (undefined && true))");
 
     identical!(deno_10668_2, "console.log(null && (undefined ?? true))");
+
+    identical!(minifier_003, "(four ** one) ** two");
+
+    identical!(minifier_004, "(void 0)(0)");
+
+    identical!(issue_1781, "const n = ~~(Math.PI * 10)");
+
+    identical!(issue_1789, "+(+1 / 4)");
 }
